/******************** (C) COPYRIGHT 2005 STMicroelectronics ********************
* File Name          : 91x_denet.c
* Author             : MCD Application Team
* Date First Issued  : 24/10/2005 :  V1.0
* Description        : This file provides all the MAC Ethernet software functions.
**********************************************************************************
* History:
* 24/10/2005 :  V1.0
**********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
* CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
* OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
* OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
* CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*********************************************************************************/

/* Standard include ----------------------------------------------------------*/
#include "91x_denet.h"


/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
typedef struct
{
  u32 DMA_Addr;
  u32 DMA_NextAddr;
  FunctionalState  WrapEn;
  FunctionalState  NpolEn;
  FunctionalState  FixAddr;
}DMA_Desciptor;


/* DENET Init structure type define */
typedef struct
{
  u8  DENET_TrDirection;
  u8  DENET_TxBurstSize;
  u8  DENET_RxBurstSize;
  u8  DENET_EntryTrig;
  u16 DENET_DfetchDly;
  u16 DENET_AddrWrap;
  u16 DENET_TimeOut;
  DMA_Descriptor DENET_TrDescriptor;
  FunctionalState  DENET_LoopBackEn;
  FunctionalState  DENET_DelayEn;
  FunctionalState  DENET_NxtEn;
  FunctionalState  DAMC_ContEn;
}MAC_InitTypeDef ;
/* Private define ------------------------------------------------------------*/
// Burst Size Mask
#define RxIncr16_Mask 0xFFFFFF3F
#define RxIncr81_Mask 0xFFFFFF7F
#define RxIncr82_Mask 0x40
#define RxIncr41_Mask 0xFFFFFFBF
#define RxIncr42_Mask 0x80
#define RxSingle_Mask 0xC0
#define TxIncr16_Mask 0xFFFFFFCF
#define TxIncr81_Mask 0xFFFFFFDF
#define TxIncr82_Mask 0x10
#define TxIncr41_Mask 0xFFFFFFEF
#define TxIncr42_Mask 0x20
#define TxSingle_Mask 0x30

// LoopBack Masks
#define LoopBackEn_Mask  0x02
#define LoopBackDis_Mask 0xFFFFFFFD

//Delay Masks
#define DelayEn_Mask  0x8000
#define DelayDis_Mask 0xFFFF7FFF

//Next DEscriptor Fetch Mask
#define NextEn_Mask  0x4000
#define NextDis_Mask 0xFFFFBFFF

//Continous mode Masks
#define ContEn_Mask  0x1000
#define ContDis_Mask 0xFFFFEFFF

//Increment DMA address masks
#define FixAddrEn_Mask  0x02

//Wrap enable mask
#define WrapEn_Mask 0x01

//Address Polling mask
#define PolEn_Mask 0x01

//Status register address
#define RxDSR_Addr 0x10
#define TxDSR_Addr 0x30
#define RxDFIFOSR_Addr 0x2C
#define TxDFIFOSR_Addr 0x4C

//DENET_GetTxRxDataConfig field values
#define TX_FifoSize            0xff
#define TX_IODataWidth         0xfe
#define TX_ChannelStatus       0xfd
#define RX_FifoSize            0xfc
#define RX_IODataWidth         0xfb
#define RX_ChannelStatus       0xfa
#define TX_MaxBurstSize        0xf9  	
#define RX_MaxBurstSize        0xf8


/* Private function prototypes -----------------------------------------------*/
/* Private variables----------------------------------------------------------*/
u32 tmp=0x0;
/* Interface functions -------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/******************************************************************************
* Function Name  : MAC_Init
* Description    : Initializes the MAC unit according to the specified parameters
*                  in the MAC_InitTypeDef structure.
* Input          : MAC_InitStruct: pointer to a MAC_InitTypeDef structure that
*                  contains the configuration information for the MAC peripheral.
* Output         : None
* Return         : None
******************************************************************************/
void DENET_Init(DENET_InitTypeDef* DENET_InitStruct)
{
  /* Sets Tx Max Burst Size  */
 if (DENET_InitStruct ->DENET_TxBurstSize == DENET_Incr16)
  {
   DENET->DSCR &=RxIncr16_Mask;
  }

 if (DENET_InitStruct ->DENET_TxBurstSize == DENET_Incr8)
  {
   DENET->DSCR &= RxIncr81_Mask;
   DENET->DSCR |= RxIncr82_Mask;
  }

 if (DENET_InitStruct ->DENET_TxBurstSize == DENET_Incr4)
  {
   DENET->DSCR &=RxIncr41_Mask;
   DENET->DSCR |=RxIncr42_Mask;
  }

 if (DENET_InitStruct ->DENET_TxBurstSize == DENET_Single)
  {
   DENET->DSCR |= RxSingle_Mask ;
  }


 /* Sets Rx Max Burst Size  */
 if (DENET_InitStruct ->DENET_RxBurstSize == DENET_Incr16)
  {
   DENET->DSCR &=TxIncr16_Mask  ;
  }

 if (DENET_InitStruct ->DENET_RxBurstSize == DENET_Incr8)
  {
   DENET->DSCR &=TxIncr81_Mask ;
   DENET->DSCR |=TxIncr82_Mask ;
  }

 if (DENET_InitStruct ->DENET_RxBurstSize == DENET_Incr4)
  {
   DENET->DSCR &=TxIncr41_Mask;
   DENET->DSCR |=TxIncr42_Mask ;
  }

 if (DENET_InitStruct ->DENET_RxBurstSize == DENET_Single)
  {
   DENET->DSCR |=TxSingle_Mask ;
  }

 /* Enable or disable loopback mode */
 if (DENET_InitStruct ->DENET_LoopBackEn == ENABLE)
  {
   DENET ->DSCR |=LoopBackEn_Mask;
  }
 if (DENET_InitStruct ->DENET_LoopBackEn == DISABLE)
  {
   DENET ->DSCR &=LoopBackDis_Mask;
  }

 /* Sets Fetch Delay */
 if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
 {
  DENET->RxDSR &=0xFF00FFFF;
  tmp=((tmp |DENET_InitStruct->DENET_DfetchDly)<<8);
  DENET->RxDSR|=tmp;
  tmp=0x0;
 }

 if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
 {
  DENET->TxDSR &=0xFF00FFFF;
  tmp=((tmp | DENET_InitStruct->DENET_DfetchDly)<<8);
  DENET->TxDSR|=tmp;
  tmp=0x0;
 }

 /*Sets DMA address counter wrap */
 if (DENET_InitStruct ->DENET_TrDirection== DENET_RX)
  {
   DENET->RxDCR&=0x3FFFFFF;
   tmp= (tmp | (DENET_InitStruct->DENET_AddrWrap)<<16) & 0xFFC0FFFF ;
   DENET->RxDCR|=tmp;
   tmp=0x0;
  }
  if (DENET_InitStruct ->DENET_TrDirection== DENET_TX)
  {
   DENET->TxDCR&=0x3FFFFFF;
  tmp= (tmp | (DENET_InitStruct->DENET_AddrWrap)<<16) & 0xFFC0FFFF ;
   DENET->TxDCR|=tmp;
   tmp=0x0;
  }

 /* Sets Entry Trig Value */

 if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
  {
    DENET->RxDCR&=0xFFC1FFFF;
    tmp=((tmp |DENET_InitStruct->DENET_EntryTrig)<<17) & 0xFF3EFFFF;
    DENET->RxDCR|=tmp;
    tmp=0x0;
  }
 if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
  {
    DENET->TxDCR&=0xFFC1FFFF;
    tmp=((tmp |DENET_InitStruct->DENET_EntryTrig)<<17) & 0xFF3EFFFF;
    DENET->TxDCR|=tmp;
    tmp=0x0;
  }

/* Enables/Disables DMA trigger delay feature */
  if (DENET_InitStruct->DENET_DelayEn == ENABLE)
   {
     if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
      {
       DENET->RxDCR|=DelayEn_Mask ;	
      }
     if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
      {
       DENET->TxDCR|=DelayEn_Mask ;	
      }
   }
  if (DENET_InitStruct->DENET_DelayEn == DISABLE)
   {
     if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
      {
       DENET->RxDCR&=DelayDis_Mask ;	
      }
     if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
      {
       DENET->TxDCR&=DelayDis_Mask ;	
      }
   }


 /* Enables/Disables Next Descriptor Fetch Mode  */
 if (DENET_InitStruct->DENET_NxtEn == ENABLE)
   {
     if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
      {
       DENET->RxDCR&=NextEn_Mask ;			
      }
     if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
      {
       DENET->TxDCR|=NextEn_Mask ;	
      }
   }
 if (DENET_InitStruct->DENET_DelayEn == DISABLE)
   {
     if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
      {
       DENET->RxDCR&=NextDis_Mask ; 	
      }
     if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
      {
       DENET->TxDCR&=NextDis_Mask ; 	
      }
   }

 /* Enables/Disables Continuous Mode */

 if (DENET_InitStruct->DENET_ContEn == ENABLE)
   {
     if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
      {
       DENET->RxDCR|=ContEn_Mask ; 	
      }
     if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
      {
       DENET->TxDCR|=ContEn_Mask ; 	
      }
   }
 if (DENET_InitStruct->DENET_ContEn == DISABLE)
   {
     if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
      {
       DENET->RxDCR&=ContDis_Mask ; 	
      }
     if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
      {
       DENET->TxDCR&=ContDis_Mask ;
      }
   }


  /* Setting DMA Transfer Description */
  // Setting DMA start Address
 if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
   {
     tmp=((DENET_InitStruct->DENET_TrDescriptor.DMA_Addr)&0xFFFFFFFC);
    DENET->RxDBAR&=0x0;
    DENET->RxDBAR|=tmp;
    tmp=0x0;	
   }
 if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
   {
    tmp=((DENET_InitStruct->DENET_TrDescriptor.DMA_Addr)&0xFFFFFFFC);
    DENET->TxDBAR&=0x0;
    DENET->TxDBAR|=tmp;
    tmp=0x0;	
   }
    // Enables/Disables incrementing of DMA_ADDR
  if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
   {
    DENET->RxDBAR&=0xFFFFFFFD;
    if (DENET_InitStruct->DENET_TrDescriptor.FixAddr == ENABLE)
     {
      DENET->RxDBAR|= FixAddrEn_Mask;
     }

   }
 if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
   {
     DENET->TxDBAR&=0xFFFFFFFD;
    if (DENET_InitStruct->DENET_TrDescriptor.FixAddr == ENABLE)
     {
      DENET->TxDBAR|=FixAddrEn_Mask;
     }
    }
  //Enables/Disables Enables wrap of the DMA transfer address
  if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
   {
    DENET->RxDBAR&=0xFFFFFFFE;
    if (DENET_InitStruct->DENET_TrDescriptor.WrapEn == ENABLE)
     {
      DENET->RxDBAR|=WrapEn_Mask ;
     }
   }
  if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
   {
     DENET->TxDBAR&=0xFFFFFFFE;
    if (DENET_InitStruct->DENET_TrDescriptor.WrapEn == ENABLE)
     {
      DENET->TxDBAR|=WrapEn_Mask ;
     }
   }

  // Setting DMA next address
 if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
   {
    DENET->RxDNDAR&=0xFFFFFFFC;
    tmp=((DENET_InitStruct->DENET_TrDescriptor.DMA_NextAddr)&0xFFFFFFFC);
    DENET->RxDNDAR&=0x0;
    DENET->RxDNDAR|=tmp;
    tmp=0x0;
   }
 if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
   {
    DENET->TxDNDAR&=0xFFFFFFFC;
    tmp=((DENET_InitStruct->DENET_TrDescriptor.DMA_NextAddr)&0xFFFFFFFC);
    DENET->TxDNDAR&=0x0;
    DENET->TxDNDAR|=tmp;
    tmp=0x0;
   }

  //Enable/Disable Next Descriptor Polling
  if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
   {
    DENET->RxDBAR&=0xFFFFFFFE;
    if (DENET_InitStruct->DENET_TrDescriptor.NpolEn == ENABLE)
     {
      DENET->RxDNDAR|=PolEn_Mask ;
     }
   }
  if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
   {
     DENET->TxDNDAR&=0xFFFFFFFE;
    if (DENET_InitStruct->DENET_TrDescriptor.NpolEn == DISABLE)
     {
      DENET->TxDNDAR|=PolEn_Mask;
     }
   }

 /* Setting DENET time out value */
 if (DENET_InitStruct->DENET_TrDirection == DENET_RX)
  {
   DENET->RxDFIFOTOR&=0xFFFF0000;
   DENET->RxDFIFOTOR|=DENET_InitStruct->DENET_TimeOut;
  }
 if (DENET_InitStruct->DENET_TrDirection == DENET_TX)
  {
   DENET->TxDFIFOTOR&=0xFFFF0000;
   DENET->TxDFIFOTOR|=DENET_InitStruct->DENET_TimeOut;
  }

}




/*******************************************************************************
* Function Name  : DENET_StructInit		         		
* Description    : Initialize the DENET Init Structure parameters
* Input          : - DENET_InitStruct: pointer to a DENET_InitTypeDef structure
                     which will be initialized.
* Output         : None	
* Return         : None.						
*******************************************************************************/
void DENET_StructInit(DENET_InitTypeDef* DENET_InitStruct)
{
  DMA_Descriptor DENET_DefaultTrDescriptor;

  DENET_DefaultTrDescriptor.DMA_Addr=0x0;
  DENET_DefaultTrDescriptor.DMA_NextAddr=0x0;
  DENET_DefaultTrDescriptor.WrapEn=DISABLE;
  DENET_DefaultTrDescriptor.NpolEn=DISABLE;
  DENET_DefaultTrDescriptor.FixAddr=DISABLE;
  /* Initialize the DENET_TrDirection member Value*/
  DENET_InitStruct->DENET_TrDirection = 0x30/* TODO */;

  /* Initialize the DENET_TxBurstSize member Value*/
  DENET_InitStruct->DENET_TxBurstSize = 0x10/* TODO */;

  /* Initialize the DENET_RXBurstSize member Value*/
  DENET_InitStruct->DENET_RxBurstSize = 0x0/* TODO */;

  /* Initialize the DENET_LoopBackEn member Value*/
  DENET_InitStruct->DENET_LoopBackEn =DISABLE/* TODO */;

  /* Initialize the MAC_DfetchDly member Value */
  DENET_InitStruct->DENET_DfetchDly= 0x0;

  /* Initialize the DENET_AddrWrap member Value */
  DENET_InitStruct->DENET_AddrWrap= 0x0;

  /* Initialize the DENET_EntryTrig member value */
  DENET_InitStruct->DENET_EntryTrig= 0x0;

  /* Initialize the DENET_DelayEn member value */
  DENET_InitStruct->DENET_DelayEn= DISABLE;

  /* Initialize the DENET_NextEn member value */
  DENET_InitStruct->DENET_NxtEn= DISABLE;

  /* Initialize the DENET_ContEn member value */
  DENET_InitStruct->DENET_ContEn=DISABLE;

  /* Initialize the DENET_TrDescriptor member Value */
  DENET_InitStruct->DENET_TrDescriptor= DENET_DefaultTrDescriptor  /* TODO */;

  /* Initialize DENET_TimeOut memeber Value */
  DENET_InitStruct->DENET_TimeOut=0x0;

 }

/*******************************************************************************
* Function Name  : DENET_GetDMAMACurrentCount		         		
* Description    : return the current DMA counter.
* Input          : - bDirection : the direction of the DMA transfer (RX or TX)
* Output         : None	
* Return         :The current DMA counter (4 bytes)						
*******************************************************************************/

u16 DENET_GetDMAMACurrentCount(u8 bDirection)
{
 u16 wResult=0x0;
 if (bDirection==DENET_RX)
  {
   wResult=DENET->RxDCTR;
   wResult&=0xFFF;
   return (wResult);
  }
 if (bDirection==DENET_TX)
 {
  wResult=DENET->RxDCTR;
  wResult&=0xFFF;
  return (wResult);
 }
return 0;
}

/*******************************************************************************
* Function Name  : DENET_GetCurrentDmaAddr		         		
* Description    : Return the current address processed by the DMA..
* Input          : - bDirection : the direction of the DMA transfer (RX or TX)
* Output         : None	
* Return         : The current DMA addr (4 bytes)						
*******************************************************************************/

u32 DENET_GetCurrentDmaAddr(u8 bDirection)
{
 if (bDirection==DENET_RX)
  {
   return (DENET->RxDCAR);
  }
  if (bDirection==DENET_TX)
  {
   return (DENET->TxDCAR);
  }
return 0;
}

/*******************************************************************************
* Function Name  : DENET_Cmd		         		
* Description    : Enables DMA cell and start data fetching
* Input          : - bDirection : the direction of the DMA transfer (RX or TX)
* Output         : None	
* Return         : None						
*******************************************************************************/

void DENET_Cmd (u8 bDirection)
{
 if (bDirection==DENET_RX)
  {
   DENET->RxDSR|=0x4;
  }
 if (bDirection==DENET_TX)
 {
  DENET->TxDSR|=0x4;
 }
}

/*******************************************************************************
* Function Name  : DENET_GetDmaEntries		         		
* Description    : Return DMA entries.
* Input          : - bDirection : the direction of the DMA transfer (RX or TX)
* Output         : None	
* Return         : 1 byte DMA entries value					
*******************************************************************************/

u8 DENET_GetDmaEntries(u8 bDirection)
{
 u8 bResult=0x0;
 if (bDirection==DENET_RX)
  {
   bResult = ((DENET->RxDFIFOSR)&0x3F000000)>>24;
   return (bResult);
  }
 if (bDirection==DENET_TX)
 {
  bResult = ((DENET->TxDFIFOSR)&0x3F000000)>>24;
  return (bResult);
 }	
 return 0;
}

/*******************************************************************************
* Function Name  : DENET_GetDmaPointer		         		
* Description    : Return DMA Pointer.
* Input          : - bDirection : the direction of the DMA transfer (RX or TX)
* Output         : None	
* Return         : 1 byte DMA pointer value					
*******************************************************************************/

u8 DENET_GetDmaPointer(u8 bDirection)
{
 u8 bResult=0x0;
 if (bDirection==DENET_RX)
  {
   bResult = ((DENET->RxDFIFOSR)&0x1F0000)>>16;
   return (bResult);
  }
 if (bDirection==DENET_TX)
 {
  bResult = ((DENET->TxDFIFOSR)&0x1F0000)>>16;
  return (bResult);
 }		
return 0;
}

/*******************************************************************************
* Function Name  : DENET_GetIOPointer		         		
* Description    : Return IO Pointer.
* Input          : - bDirection : the direction of the DMA transfer (RX or TX)
* Output         : None	
* Return         : 1 byte IO pointer value					
*******************************************************************************/

u8 DENET_GetIOPointer(u8 bDirection)
{
 u8 bResult=0x0;
 if (bDirection==DENET_RX)
  {
   bResult = ((DENET->RxDFIFOSR)&0x1F00)>>8;
   return (bResult);
  }
 if (bDirection==DENET_TX)
 {
  bResult = ((DENET->TxDFIFOSR)&0x1F00)>>8;
  return (bResult);
 }		
 return 0;
}

/*******************************************************************************
* Function Name  : DENET_SoftResetCmd		         		
* Description    : DMA soft reset, holds the whole DMA_MAC and MAC-802.3 logic
*                  in reset condition. Until the Soft reset is processed no
*                  operation is allowed on the DMA_MAC or MAC-802.3 registers,
*                  except exiting reset phase.This signal has no effect on the
*                  AHB interface so, when asserted runtime, the whole DMA will
*                  be reset only when the last AHB transfer, in the AHB master
*                  queue, has been completed.
* Input          : - NewState : Enable / Disable
* Output         : None	
* Return         : None					
*******************************************************************************/

void DENET_SoftResetCmd(FunctionalState NewState)
{
 if (NewState == ENABLE)
 {
  DENET->DSCR|=0x1;
 }
 if (NewState == DISABLE)
 {
  DENET->DSCR&=0xFFFFFFFE;	
 }	
}

/*******************************************************************************s
* Function Name  : DENET_GetTxRxDataConfig		         		
* Description    : Returns the configuration concerning RX and TX FIFOs and the
*                  data organisation
* Input          : bField: the paramater which will be checked , it can be :
*                  - TX_FifoSize : The size of the TX FIFO.
*                 - TX_IODataWidth: The width of the IO bus transmit data path
*                - TX_ChannelStatus : Information about the TX channel structure
*                  - RX_FifoSize: The size of the RX FIFO
*                  - RX_IODataWidth: The width of the IO bus receive data path
*                  - RX_ChannelStatus : Information about the TX channel structure
*                  - TX_MaxBurstSize: Maximum value of defined lenght burst that
*                   TX DMA_MAC will perform on the AHB bus.
*                  - RX_MaxBurstSize: Maximum value of defined lenght burst that
*                    RX DMA_MAC will perform on the AHB bus.
* Output         : Value of the selected filed.	
* Return         : None					
*******************************************************************************/

u8 DENET_GetTxRxDataConfig(u8 bField)
{
 u8 bResult=0x0;
 switch (bField)
 {
  case TX_FifoSize :   bResult==((DENET->DSCR)&0xF0000000)>>28;
                       return bResult; 	
                       break;
  case TX_IODataWidth: bResult==((DENET->DSCR)&0xC000000)>>26;
                       return bResult; 	
                       break;
  case TX_ChannelStatus: bResult==((DENET->DSCR)&0x3000000)>>24;
                         return bResult; 	
                         break;
  case RX_FifoSize:    bResult==((DENET->DSCR)&0xF00000)>>20;
                       return bResult; 	
                       break;
  case RX_IODataWidth: bResult==((DENET->DSCR)&0xC0000)>>18;
                       return bResult; 	
                       break;
  case RX_ChannelStatus: bResult==((DENET->DSCR)&0x30000)>>16;
                       return bResult; 	
                       break;
  case TX_MaxBurstSize: bResult==((DENET->DSCR)&0xC000)>>6;
                        return bResult; 	
                        break;          	
  case RX_MaxBurstSize: bResult==((DENET->DSCR)&0x3000)>>4;
                        return bResult; 	
                        break;
  default   : return 0;        	
 }	
 return 0;
}

/*******************************************************************************
* Function Name  : DENET_ITConfig		         		
* Description    : Enables or disables the selected interrupt
* Input          : - DENET_IT : The selected interrupt to be configured.
                   - NewState : new state of the selected inerrupt.This
                     parameter can be: ENABLE or DISABLE.
* Output         : None
* Return         : None					
*******************************************************************************/

void DENET_ITConfig(u8 DENET_IT, FunctionalState NewState)
{
 if (NewState == ENABLE)
  {
   DENET->DISER|= (0x01<<DENET_IT);
  }
 if (NewState == DISABLE)
  {
   DENET->DISER&= (0xFFFFFFFE<<DENET_IT);	
  }
}

/*******************************************************************************
* Function Name  : DENET_ClearITPendingBit		         		
* Description    : Clear IT pending bit for the selected IT.
* Input          : - DENET_IT : The selected interrupt to be configured.
* Output         : None
* Return         : None					
*******************************************************************************/

void DENET_ClearITPendingBit(u8 DENET_IT)
{
 DENET->DISR|=(0x01<<DENET_IT);
}

/*******************************************************************************
* Function Name  : DENET_GetITStatus		         		
* Description    : DENET_ClearITPendingBit
* Input          : - DENET_IT : The selected interrupt to be configured.
* Output         : None
* Return         : The status of the selected IT.This status can be SET or RESET					
*******************************************************************************/

ITStatus DENET_GetITSatus(u8 DENET_IT)
{
 if ((DENET->DISR)&(0x01<<DENET_IT)!=0)
  {
   return SET;	
  }
 if ((DENET->DISR)&(0x01<<DENET_IT)==0)
  {
   return RESET;	
  }
return 0;
}

/*******************************************************************************
* Function Name  : DENET_GetFlagStatus		         		
* Description    : Checks whether the MAC flag is set or not.
* Input          : - MAC_Flag: specifies the flag to check.
* Output         : None
* Return         : The status of the selected Flag. This status can be SET or
*                  RESET.					
*******************************************************************************/

FlagStatus DENET_GetFlagSatus(u16 DENET_FLAG)
{
 if ((DENET_FLAG & 0xFF00) == RxDSR_Addr)
  {
   if ((DENET->RxDSR)&(0x1<<(DENET_FLAG & 0xFF)) != RESET)
    {
    	return SET;
    }
    else
     {
      return RESET;
     }
  }
  if ((DENET_FLAG & 0xFF00) == TxDSR_Addr)
  {
   if ((DENET->TxDSR)&(0x1<<(DENET_FLAG & 0xFF)) != RESET)
    {
    	return SET;
    }
    else
     {
      return RESET;
     }
  }
  if ((DENET_FLAG & 0xFF00) == RxDFIFOSR_Addr)
  {
   if ((DENET->RxDFIFOSR)&(0x1<<(DENET_FLAG & 0xFF)) != RESET)
    {
    	return SET;
    }
    else
     {
      return RESET;
     }
  }
  if ((DENET_FLAG & 0xFF00) == TxDFIFOSR_Addr)
  {
   if ((DENET->TxDFIFOSR)&(0x1<<(DENET_FLAG & 0xFF)) != RESET)
    {
    	return SET;
    }
    else
     {
      return RESET;
     }
  }
 return 0;
}

